#ifndef __AES_FILE_H__
#define __AES_FILE_H__

#include <string>
#include <cstdint>
#include <vector>
#include <cstdio>
#include <wx/string.h>
#include "msgbox.h"
#include <cstring>
#include "Common.h"


class AesFile
{
public:
	static const size_t KEY_SIZE = 32;
	static const size_t IV_SIZE = 16;
	static const size_t MAGIC_SIZE = IV_SIZE;

	struct KeyFile
	{
		std::string fname;

		uint8_t magic[MAGIC_SIZE];
		uint8_t key[KEY_SIZE];
		uint8_t iv[IV_SIZE];

		inline KeyFile()
		{
			memset(magic, 0, sizeof(magic));
			memset(key, 0, sizeof(key));
			memset(iv, 0, sizeof(iv));
		}

		inline void print()
		{
			showDebugMsg(wxString(_T("fname=")) + wxString(fname) + wxString(_T("\n\n")) +
				wxString(_T("magic=")) + arrayToHex(magic, MAGIC_SIZE) + wxString(_T("\n\n")) +
				wxString(_T("key=")) + arrayToHex(key, KEY_SIZE) + wxString(_T("\n\n")) +
				wxString(_T("iv=")) + arrayToHex(iv, IV_SIZE));
		}
	};

	explicit AesFile(const KeyFile &keyFile);
	virtual ~AesFile();

	std::vector<uint8_t> encryptFileData(const std::vector<uint8_t> &plainData);
	bool decryptFileData(const std::vector<uint8_t> &cryptedData, std::vector<uint8_t> &plainData);

protected:

	static void encryptAES256(const uint8_t key[32], const uint8_t iv[16], const uint8_t *data16, size_t data_size16, uint8_t *out16);
	static void decryptAES256(const uint8_t key[32], const uint8_t iv[16], const uint8_t *data16, size_t data_size16, uint8_t *out16);

	static std::vector<uint8_t> encryptData(const uint8_t key[32], const uint8_t iv[16], const std::vector<uint8_t> &plainData);
	static bool decryptData(const uint8_t key[32], const uint8_t iv[16], const std::vector<uint8_t> &cryptedData, size_t realDataSize, std::vector<uint8_t> &plainData);

	static std::vector<uint8_t> encryptFileData(const KeyFile &keyFile, const std::vector<uint8_t> &plainData);
	static bool decryptFileData(const KeyFile &keyFile, const std::vector<uint8_t> &cryptedData, std::vector<uint8_t> &plainData);

private:
	KeyFile mKeyFile;
};

#endif // !__AES_FILE_H__
