#ifndef __CDB_FIELD_H__
#define __CDB_FIELD_H__


#include <cstdint>
#include <wx/string.h>
#include <vector>
#include "Common.h"


class CDBFieldInteger final
{
public:
	explicit CDBFieldInteger();
	explicit CDBFieldInteger(int64_t value);
	virtual ~CDBFieldInteger();

	bool isNull() const;
	void setNull();

	void setValue(int64_t value);
	int64_t getValue(int64_t def = 0) const;

	std::vector<uint8_t> toRawData() const;
	static CDBFieldInteger fromRawData(const std::vector<uint8_t> &rawData);

	inline wxString toString()
	{
		wxString res = _T("(");
		
		if (isNull())
			res += _T("NULL");
		else
		{
			res += _T("NOT NULL, ");
			res += int64ToHex(getValue());
		}

		res += _T(")");
		
		return res;
	}

private:
	bool mNull;
	int64_t mValue;
};

inline bool operator==(const CDBFieldInteger &a, const CDBFieldInteger &b)
{
	if (a.isNull() && b.isNull())
		return true;

	if (!a.isNull() && !b.isNull() && a.getValue() == b.getValue())
		return true;

	return false;
}



class CDBFieldText final
{
public:
	explicit CDBFieldText();
	explicit CDBFieldText(wxString value);
	virtual ~CDBFieldText();

	bool isNull() const;
	void setNull();

	void setValue(wxString value);
	wxString getValue(const wxString &def = wxString()) const;

	std::vector<uint8_t> toRawData() const;
	static CDBFieldText fromRawData(const std::vector<uint8_t> &rawData);

	inline wxString toString()
	{
		wxString res = _T("(");

		if (isNull())
			res += _T("NULL");
		else
		{
			res += _T("NOT NULL, [");
			res += getValue();
			res += _T("]");
		}

		res += _T(")");

		return res;
	}

private:
	bool mNull;
	wxString mValue;
};

inline bool operator==(const CDBFieldText &a, const CDBFieldText &b)
{
	if (a.isNull() && b.isNull())
		return true;

	if (!a.isNull() && !b.isNull() && a.getValue() == b.getValue())
		return true;

	return false;
}



#endif //__CDB_FIELD_H__
