#include "CDBTableFields.h"


enum RawTableLineType { RTLT_UNKNOWN = 0, RTLT_SERVER = 0xEE11, RTLT_FILE = 0xEE22, RTLT_PATH = 0xEE33};


#pragma pack(push, 1)
struct TableLineHeader
{
	uint32_t line_type;
	uint32_t count_fields;
};
#pragma pack(pop)


std::vector<uint8_t> packRawFieldsToRawData(const std::vector<std::vector<uint8_t> > &rawFields, RawTableLineType lineType)
{
	const size_t fields_count = rawFields.size();

	std::vector<uint8_t> rawData(sizeof(TableLineHeader), 0);

	TableLineHeader* pHeader = (TableLineHeader*)rawData.data();
	pHeader->line_type = (uint32_t)lineType;
	pHeader->count_fields = (uint32_t)fields_count;

	if (fields_count > 0)
	{
		for (const std::vector<uint8_t> &field : rawFields)
		{
			const size_t field_size = field.size();

			std::vector<uint8_t> packed_field(sizeof(uint32_t), 0);
			uint32_t* pSizeField = (uint32_t*)packed_field.data();
			*pSizeField = (uint32_t)field_size;

			if (field_size > 0)
				packed_field.insert(packed_field.end(), field.begin(), field.end());

			rawData.insert(rawData.end(), packed_field.begin(), packed_field.end());
		}
	}

	return rawData;
}

std::vector<std::vector<uint8_t> > unpackRawFieldsFromRawData(const std::vector<uint8_t> &rawData, RawTableLineType lineType)
{
	std::vector<std::vector<uint8_t>> rawFields;

	if (rawData.size() >= sizeof(TableLineHeader))
	{
		const TableLineHeader* pHeader = (const TableLineHeader*)rawData.data();
		const uint32_t ltype = (uint32_t)pHeader->line_type;
		const size_t fields_count = (size_t)pHeader->count_fields;
		
		if (ltype == (uint32_t)lineType)
		{
			if (fields_count > 0)
			{
				size_t cur_off = sizeof(TableLineHeader);
				size_t rest_bytes = rawData.size() - sizeof(TableLineHeader);

				for (size_t i = 0; i < fields_count; ++i)
				{
					if (rest_bytes >= sizeof(uint32_t))
					{
						const uint32_t* pFieldSize = (const uint32_t*)(rawData.data() + cur_off);
						const size_t field_size = (size_t)(*pFieldSize);

						cur_off += sizeof(uint32_t);
						rest_bytes -= sizeof(uint32_t);

						if (field_size > 0)
						{
							if (rest_bytes >= field_size)
							{
								std::vector<uint8_t> rawField(field_size, 0);
								memcpy(rawField.data(), rawData.data() + cur_off, field_size);

								cur_off += field_size;
								rest_bytes -= field_size;

								rawFields.push_back(rawField);
							}
							else
								break;
						}
						else
							rawFields.push_back(std::vector<uint8_t>());
					}
					else
						break;
				}
			}
		}
	}

	return rawFields;
}



/*
Column cry_server_table_columns[] = {
{ _T("id"), Column_type::integer, Column_flags::not_null | Column_flags::autoincrement },

{ _T("host"), Column_type::text, Column_flags::not_null },
{ _T("port"), Column_type::integer, 0 },
{ _T("user"), Column_type::text, 0 },
{ _T("password"), Column_type::text, 0 },
{ _T("account"), Column_type::text, 0 },
{ _T("protocol"), Column_type::integer, 0 },
{ _T("type"), Column_type::integer, 0 },
{ _T("logontype"), Column_type::integer, 0 },
{ _T("timezone_offset"), Column_type::integer, 0 },
{ _T("transfer_mode"), Column_type::text, 0 },
{ _T("max_connections"), Column_type::integer, 0 },
{ _T("encoding"), Column_type::text, 0 },
{ _T("bypass_proxy"), Column_type::integer, 0 },
{ _T("post_login_commands"), Column_type::text, 0 },
{ _T("name"), Column_type::text, 0 }
};
*/

// CDBTableServerLine

std::vector<uint8_t> CDBTableServerLine::toRawData() const
{
	std::vector<std::vector<uint8_t> > rawFields;

	rawFields.push_back(host.toRawData());
	rawFields.push_back(port.toRawData());
	rawFields.push_back(user.toRawData());
	rawFields.push_back(password.toRawData());
	rawFields.push_back(account.toRawData());
	rawFields.push_back(protocol.toRawData());
	rawFields.push_back(type.toRawData());
	rawFields.push_back(logontype.toRawData());
	rawFields.push_back(timezone_offset.toRawData());
	rawFields.push_back(transfer_mode.toRawData());
	rawFields.push_back(max_connections.toRawData());
	rawFields.push_back(encoding.toRawData());
	rawFields.push_back(bypass_proxy.toRawData());
	rawFields.push_back(post_login_commands.toRawData());
	rawFields.push_back(name.toRawData());

	return packRawFieldsToRawData(rawFields, RTLT_SERVER);
}

CDBTableServerLine CDBTableServerLine::fromRawData(const std::vector<uint8_t>& rawData)
{
	const std::vector<std::vector<uint8_t> > rawFields = unpackRawFieldsFromRawData(rawData, RTLT_SERVER);

	CDBTableServerLine res;

	if (rawFields.size() == 15)
	{
		res.host = CDBFieldText::fromRawData(rawFields[0]);
		res.port = CDBFieldInteger::fromRawData(rawFields[1]);
		res.user = CDBFieldText::fromRawData(rawFields[2]);
		res.password = CDBFieldText::fromRawData(rawFields[3]);
		res.account = CDBFieldText::fromRawData(rawFields[4]);
		res.protocol = CDBFieldInteger::fromRawData(rawFields[5]);
		res.type = CDBFieldInteger::fromRawData(rawFields[6]);
		res.logontype = CDBFieldInteger::fromRawData(rawFields[7]);
		res.timezone_offset = CDBFieldInteger::fromRawData(rawFields[8]);
		res.transfer_mode = CDBFieldText::fromRawData(rawFields[9]);
		res.max_connections = CDBFieldInteger::fromRawData(rawFields[10]);
		res.encoding = CDBFieldText::fromRawData(rawFields[11]);
		res.bypass_proxy = CDBFieldInteger::fromRawData(rawFields[12]);
		res.post_login_commands = CDBFieldText::fromRawData(rawFields[13]);
		res.name = CDBFieldText::fromRawData(rawFields[14]);
	}

	return res;
}





// CDBTableFileLine





/*
Column cry_file_table_columns[] = {
{ _T("id"), Column_type::integer, Column_flags::not_null | Column_flags::autoincrement },
{ _T("server"), Column_type::integer, Column_flags::not_null },
{ _T("source_file"), Column_type::text, 0 },
{ _T("target_file"), Column_type::text, 0 },
{ _T("local_path"), Column_type::integer, 0 },
{ _T("remote_path"), Column_type::integer, 0 },
{ _T("download"), Column_type::integer, Column_flags::not_null },
{ _T("size"), Column_type::integer, 0 },
{ _T("error_count"), Column_type::integer, 0 },
{ _T("priority"), Column_type::integer, 0 },
{ _T("ascii_file"), Column_type::integer, 0 },
{ _T("default_exists_action"), Column_type::integer, 0 }
};
*/

std::vector<uint8_t> CDBTableFileLine::toRawData() const
{
	std::vector<std::vector<uint8_t> > rawFields;

	rawFields.push_back(server_id.toRawData());
	rawFields.push_back(source_file.toRawData());
	rawFields.push_back(target_file.toRawData());
	rawFields.push_back(local_path_id.toRawData());
	rawFields.push_back(remote_path_id.toRawData());
	rawFields.push_back(download.toRawData());
	rawFields.push_back(size.toRawData());
	rawFields.push_back(error_count.toRawData());
	rawFields.push_back(priority.toRawData());
	rawFields.push_back(ascii_file.toRawData());
	rawFields.push_back(default_exists_action.toRawData());

	return packRawFieldsToRawData(rawFields, RTLT_FILE);
}

CDBTableFileLine CDBTableFileLine::fromRawData(const std::vector<uint8_t>& rawData)
{
	const std::vector<std::vector<uint8_t> > rawFields = unpackRawFieldsFromRawData(rawData, RTLT_FILE);

	CDBTableFileLine res;

	if (rawFields.size() == 11)
	{
		res.server_id = CDBFieldInteger::fromRawData(rawFields[0]);
		res.source_file = CDBFieldText::fromRawData(rawFields[1]);
		res.target_file = CDBFieldText::fromRawData(rawFields[2]);
		res.local_path_id = CDBFieldInteger::fromRawData(rawFields[3]);
		res.remote_path_id = CDBFieldInteger::fromRawData(rawFields[4]);
		res.download = CDBFieldInteger::fromRawData(rawFields[5]);
		res.size = CDBFieldInteger::fromRawData(rawFields[6]);
		res.error_count = CDBFieldInteger::fromRawData(rawFields[7]);
		res.priority = CDBFieldInteger::fromRawData(rawFields[8]);
		res.ascii_file = CDBFieldInteger::fromRawData(rawFields[9]);
		res.default_exists_action = CDBFieldInteger::fromRawData(rawFields[10]);
	}

	return res;
}





// CDBTablePathLine





/*
Column cry_path_table_columns[] = {
{ _T("id"), Column_type::integer, Column_flags::not_null | Column_flags::autoincrement },

{ _T("path"), Column_type::text, Column_flags::not_null }
};
*/

std::vector<uint8_t> CDBTablePathLine::toRawData() const
{
	std::vector<std::vector<uint8_t> > rawFields;

	rawFields.push_back(path.toRawData());

	return packRawFieldsToRawData(rawFields, RTLT_PATH);
}

CDBTablePathLine CDBTablePathLine::fromRawData(const std::vector<uint8_t>& rawData)
{
	const std::vector<std::vector<uint8_t> > rawFields = unpackRawFieldsFromRawData(rawData, RTLT_PATH);

	CDBTablePathLine res;

	if (rawFields.size() == 1)
	{
		res.path = CDBFieldText::fromRawData(rawFields[0]);
	}

	return res;
}
