#ifndef __CDB_TABLES_H__
#define __CDB_TABLES_H__


#include "CDBTableFields.h"
#include <vector>
#include <cstdint>
#include <map>




template <class TTableLine>
class CDBBaseTable
{
public:
	typedef typename std::pair<int64_t, TTableLine> TIDLine;

	explicit CDBBaseTable()
		: mIdCounter(0)
	{
	}

	virtual ~CDBBaseTable() {}
	
	std::vector<TIDLine> getAllLines() const
	{
		std::vector<TIDLine> res;

		typedef typename std::map<int64_t, TTableLine>::const_iterator ConstIterator;
		for (ConstIterator it = mMapLines.begin(); it != mMapLines.end(); ++it)
		{
			const int64_t id = it->first;
			const TTableLine line = it->second;

			TIDLine idLine;
			idLine.first = id;
			idLine.second = line;

			res.push_back(idLine);
		}

		return res;
	}

	int64_t addLine(const TTableLine &line)
	{
		const int64_t id = nextIDCounter();
		mMapLines[id] = line;
		return id;
	}

protected:
	int64_t getIDCounter() const
	{
		return mIdCounter;
	}

	void setIDCounter(int64_t idCounter)
	{
		mIdCounter = idCounter;
	}

	int64_t nextIDCounter()
	{
		setIDCounter(getIDCounter() + 1);
		return getIDCounter();
	}

protected:
	std::map<int64_t, TTableLine> mMapLines;

private:
	int64_t mIdCounter;
};


class CDBTableServer : public CDBBaseTable<CDBTableServerLine>
{
public:
	std::vector<uint8_t> toRawData() const;
	static CDBTableServer fromRawData(const std::vector<uint8_t> &rawData);
};


class CDBTableFile : public CDBBaseTable<CDBTableFileLine>
{
public:
	std::vector<uint8_t> toRawData() const;
	static CDBTableFile fromRawData(const std::vector<uint8_t> &rawData);
};


class CDBTablePath : public CDBBaseTable<CDBTablePathLine>
{
public:
	std::vector<uint8_t> toRawData() const;
	static CDBTablePath fromRawData(const std::vector<uint8_t> &rawData);
};


#endif //__CDB_TABLES_H__
