#include "CMasterPassChangeDialog.h"
#include "Common.h"
#include <wx/string.h>



CMasterPassChangeDialog::CMasterPassChangeDialog()
	: wxDialog(nullptr, wxID_ANY, wxString(_T("Change Master Password.")), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxSTAY_ON_TOP),
	  mWxTextCtrlOldPass(nullptr), mWxTextCtrlNewPassFirst(nullptr), mWxTextCtrlNewPassSecond(nullptr),
	  mOldPass(), mNewPass()
{
	const int editSize = 350;
	mWxTextCtrlOldPass = new wxTextCtrl(this, wxID_ANY, wxString(), wxDefaultPosition, wxSize(editSize, -1), wxTE_PASSWORD);
	mWxTextCtrlNewPassFirst = new wxTextCtrl(this, wxID_ANY, wxString(), wxDefaultPosition, wxSize(editSize, -1), wxTE_PASSWORD);
	mWxTextCtrlNewPassSecond = new wxTextCtrl(this, wxID_ANY, wxString(), wxDefaultPosition, wxSize(editSize, -1), wxTE_PASSWORD);

	wxButton* okButton = new wxButton(this, wxID_OK);
	okButton->SetDefault();
	wxButton* cancelButton = new wxButton(this, wxID_CANCEL);


	wxStaticText* textEnterOldPass = new wxStaticText(this, wxID_ANY, "Enter Old Password:");
	
	wxStaticText* textPlease = new wxStaticText(this, wxID_ANY, "Please choose the New Master Password to encrypt your FTP login details.");
	wxStaticText* textEnterNewPass = new wxStaticText(this, wxID_ANY, "Enter New Password:");
	wxStaticText* textConfirmNewPass = new wxStaticText(this, wxID_ANY, "Confirm New Password:");

	wxBoxSizer *h = new wxBoxSizer(wxVERTICAL);
	h->Add(textEnterOldPass, 0, wxALL, 10);
	h->Add(mWxTextCtrlOldPass, 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10);

	h->Add(textPlease, 0, wxALL, 10);

	h->Add(textEnterNewPass, 0, wxALL, 10);
	h->Add(mWxTextCtrlNewPassFirst, 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10);

	h->Add(textConfirmNewPass, 0, wxALL, 10);
	h->Add(mWxTextCtrlNewPassSecond, 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10);

	wxBoxSizer *hor = new wxBoxSizer(wxHORIZONTAL);
	hor->Add(okButton, 0, wxALL | wxALIGN_CENTRE, 10);
	hor->Add(cancelButton, 0, wxALL | wxALIGN_CENTRE, 10);

	h->Add(hor, 0, wxALL | wxALIGN_CENTRE, 10);

	SetSizerAndFit(h);

	Bind(wxEVT_COMMAND_BUTTON_CLICKED, &CMasterPassChangeDialog::onOk, this, okButton->GetId());
	Bind(wxEVT_COMMAND_BUTTON_CLICKED, &CMasterPassChangeDialog::onCancel, this, cancelButton->GetId());

	Center();
}

CMasterPassChangeDialog::~CMasterPassChangeDialog()
{
	Destroy();
}

void CMasterPassChangeDialog::onCancel(wxCommandEvent & WXUNUSED(pEvent))
{
	mOldPass.Clear();
	mNewPass.Clear();

	EndModal(wxID_CANCEL);
}

void CMasterPassChangeDialog::onOk(wxCommandEvent & WXUNUSED(pEvent))
{
	const wxString oldPass = mWxTextCtrlOldPass->GetValue();
	const wxString firstPass = mWxTextCtrlNewPassFirst->GetValue();
	const wxString secondPass = mWxTextCtrlNewPassSecond->GetValue();

	if (oldPass.Length() < 8)
	{
		wxMessageBoxEx(_T("The Old Password must have length at least 8 symbols."), _T("Change Master Password"), wxOK | wxCENTRE | wxICON_WARNING, this);
	}
	else
	{
		if (firstPass.Length() < 8)
		{
			wxMessageBoxEx(_T("The New Password must have length at least 8 symbols."), _T("Change Master Password"), wxOK | wxCENTRE | wxICON_WARNING, this);
		}
		else
		{
			if (firstPass == secondPass)
			{
				mOldPass = oldPass;
				mNewPass = firstPass;
				EndModal(wxID_OK);
			}
			else
				wxMessageBoxEx(_T("The New Passwords are not the same."), _T("Change Master Password"), wxOK | wxCENTRE | wxICON_WARNING, this);
		}
	}
}

wxString CMasterPassChangeDialog::getOldPass() const
{
	return mOldPass;
}

wxString CMasterPassChangeDialog::getNewPass() const
{
	return mNewPass;
}

bool showDlgChangeMasterPassword(wxString & outOldPass, wxString & outNewPass)
{
	outOldPass.Clear();
	outNewPass.Clear();

	CMasterPassChangeDialog dlgChangePass;
	if (dlgChangePass.ShowModal() == wxID_OK)
	{
		outOldPass = dlgChangePass.getOldPass();
		outNewPass = dlgChangePass.getNewPass();

		return true;
	}

	return false;
}
