#include "CMasterPassCreateDialog.h"
#include "Common.h"
#include <wx/string.h>



CMasterPassCreateDialog::CMasterPassCreateDialog()
	: wxDialog(nullptr, wxID_ANY, wxString(_T("Enter New Master Password.")), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxSTAY_ON_TOP),
	  mWxTextCtrlPassFirst(nullptr), mWxTextCtrlPassSecond(nullptr), mNewPass()
{
	const int editSize = 350;
	mWxTextCtrlPassFirst = new wxTextCtrl(this, wxID_ANY, wxString(), wxDefaultPosition, wxSize(editSize, -1), wxTE_PASSWORD);
	mWxTextCtrlPassSecond = new wxTextCtrl(this, wxID_ANY, wxString(), wxDefaultPosition, wxSize(editSize, -1), wxTE_PASSWORD);

	wxButton* okButton = new wxButton(this, wxID_OK);
	okButton->SetDefault();
	wxButton* cancelButton = new wxButton(this, wxID_CANCEL);


	wxStaticText* textPlease = new wxStaticText(this, wxID_ANY, "Please choose a master password to encrypt your FTP login details.");
	wxStaticText* textEnterPass = new wxStaticText(this, wxID_ANY, "Enter password:");
	wxStaticText* textConfirmPass = new wxStaticText(this, wxID_ANY, "Confirm password:");

	wxBoxSizer *h = new wxBoxSizer(wxVERTICAL);
	h->Add(textPlease, 0, wxALL, 10);
	
	h->Add(textEnterPass, 0, wxALL, 10);
	h->Add(mWxTextCtrlPassFirst, 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10);
	
	h->Add(textConfirmPass, 0, wxALL, 10);
	h->Add(mWxTextCtrlPassSecond, 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10);
	
	wxBoxSizer *hor = new wxBoxSizer(wxHORIZONTAL);
	hor->Add(okButton, 0, wxALL | wxALIGN_CENTRE, 10);
	hor->Add(cancelButton, 0, wxALL | wxALIGN_CENTRE, 10);

	h->Add(hor, 0, wxALL | wxALIGN_CENTRE, 10);

	SetSizerAndFit(h);

	Bind(wxEVT_COMMAND_BUTTON_CLICKED, &CMasterPassCreateDialog::onOk, this, okButton->GetId());
	Bind(wxEVT_COMMAND_BUTTON_CLICKED, &CMasterPassCreateDialog::onCancel, this, cancelButton->GetId());	

	Center();
}

CMasterPassCreateDialog::~CMasterPassCreateDialog()
{
	Destroy();
}

void CMasterPassCreateDialog::onCancel(wxCommandEvent & WXUNUSED(pEvent))
{
	EndModal(wxID_CANCEL);
}

void CMasterPassCreateDialog::onOk(wxCommandEvent & WXUNUSED(pEvent))
{
	const wxString firstPass = mWxTextCtrlPassFirst->GetValue();
	const wxString secondPass = mWxTextCtrlPassSecond->GetValue();

	if (firstPass.Length() < 8)
	{
		wxMessageBoxEx(_T("The password must have length at least 8 symbols."), _T("New Master Password"), wxOK | wxCENTRE | wxICON_WARNING, this);
	}
	else
	{
		if (firstPass == secondPass)
		{
			mNewPass = firstPass;
			EndModal(wxID_OK);
		}
		else
			wxMessageBoxEx(_T("The passwords are not the same."), _T("New Master Password"), wxOK | wxCENTRE | wxICON_WARNING, this);
	}
}

wxString CMasterPassCreateDialog::getNewPass() const
{
	return mNewPass;
}



bool showDlgNewMasterPassword(wxString &outPass)
{
	outPass.Clear();

	CMasterPassCreateDialog dlgCreatePass;
	if (dlgCreatePass.ShowModal() == wxID_OK)
	{
		outPass = dlgCreatePass.getNewPass();
		return true;
	}

	return false;
}
