#include "CMasterPassEnterDialog.h"

#include "Common.h"
#include <wx/string.h>


CMasterPassEnterDialog::CMasterPassEnterDialog()
	: wxDialog(nullptr, wxID_ANY, wxString(_T("Enter Master Password.")), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxSTAY_ON_TOP),
	  mWxTextCtrlPass(nullptr), mPass(), mIsChangePass(false)
{
	const int editSize = 350;
	mWxTextCtrlPass = new wxTextCtrl(this, wxID_ANY, wxString(), wxDefaultPosition, wxSize(editSize, -1), wxTE_PASSWORD);

	wxButton* okButton = new wxButton(this, wxID_OK);
	okButton->SetDefault();
	wxButton* cancelButton = new wxButton(this, wxID_CANCEL);
	wxButton* changePassButton = new wxButton(this, wxID_ANY, "Change Password");


	wxStaticText* textPlease = new wxStaticText(this, wxID_ANY, "Please enter the master password to decrypt your FTP login details.");
	wxStaticText* textEnterPass = new wxStaticText(this, wxID_ANY, "Enter password:");

	wxBoxSizer *h = new wxBoxSizer(wxVERTICAL);
	h->Add(textPlease, 0, wxALL, 10);

	h->Add(textEnterPass, 0, wxALL, 10);
	h->Add(mWxTextCtrlPass, 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10);

	wxBoxSizer *hor = new wxBoxSizer(wxHORIZONTAL);
	hor->Add(okButton, 0, wxALL | wxALIGN_LEFT, 10);
	hor->Add(cancelButton, 0, wxALL | wxALIGN_LEFT, 10);
	hor->Add(changePassButton, 0, wxALL | wxALIGN_RIGHT, 10);

	h->Add(hor, 0, wxALL | wxALIGN_CENTRE, 10);

	SetSizerAndFit(h);

	Bind(wxEVT_COMMAND_BUTTON_CLICKED, &CMasterPassEnterDialog::onOk, this, okButton->GetId());
	Bind(wxEVT_COMMAND_BUTTON_CLICKED, &CMasterPassEnterDialog::onCancel, this, cancelButton->GetId());
	Bind(wxEVT_COMMAND_BUTTON_CLICKED, &CMasterPassEnterDialog::onChangeMasterPassword, this, changePassButton->GetId());

	Center();
}

CMasterPassEnterDialog::~CMasterPassEnterDialog()
{
	Destroy();
}

void CMasterPassEnterDialog::onCancel(wxCommandEvent & WXUNUSED(pEvent))
{
	mPass.Clear();
	mIsChangePass = false;
	EndModal(wxID_CANCEL);
}

void CMasterPassEnterDialog::onOk(wxCommandEvent & WXUNUSED(pEvent))
{
	const wxString pass = mWxTextCtrlPass->GetValue();

	if (pass.Length() < 8)
	{
		wxMessageBoxEx(_T("The password must have length at least 8 symbols."), _T("Enter Master Password"), wxOK | wxCENTRE | wxICON_WARNING, this);
		mPass.Clear();
		mIsChangePass = false;
	}
	else
	{
		mPass = pass;
		mIsChangePass = false;
		EndModal(wxID_OK);
	}
}

void CMasterPassEnterDialog::onChangeMasterPassword(wxCommandEvent & WXUNUSED(pEvent))
{
	mPass.Clear();
	mIsChangePass = true;
	EndModal(wxID_OK);
}

wxString CMasterPassEnterDialog::getPass() const
{
	return mPass;
}

bool CMasterPassEnterDialog::isChangePass() const
{
	return mIsChangePass;
}

bool showDlgEnterMasterPassword(wxString & outPass, bool & isChangeMasterPass)
{
	outPass.Clear();
	isChangeMasterPass = false;

	CMasterPassEnterDialog dlg;
	if (dlg.ShowModal() == wxID_OK)
	{
		if (dlg.isChangePass())
		{
			outPass.Clear();
			isChangeMasterPass = true;
		}
		else
		{
			outPass = dlg.getPass();
			isChangeMasterPass = false;
		}

		return true;
	}

	return false;
}
