#ifndef __CRYPT_FILES_H___
#define __CRYPT_FILES_H___

#include <string>
#include <cstdint>
#include <vector>
#include <cstdio>
#include <wx/string.h>
#include "msgbox.h"
#include <cstring>
#include "AesFile.h"
#include "ShaMasterPass.h"
#include "Common.h"


enum class CryptFileType { FileZillaXml, SiteManagerXml, RecentServersXml, QueueSqlite3, QueueXml, BookmarksXml, TrustedcertsXml};

class CryptFiles  //Singleton
{
private:
	CryptFiles() {}
	CryptFiles(const CryptFiles &);
	CryptFiles& operator=(const CryptFiles&);

public:
	static CryptFiles& instance()
	{
		static CryptFiles singletonInstance;
		return singletonInstance;
	}

	static bool loadAllDataFromFile(CryptFileType cfType, std::vector<uint8_t> &data);
	static bool saveAllDataToFile(CryptFileType cfType, const std::vector<uint8_t> &data);

	std::vector<uint8_t> encryptFileData(const std::vector<uint8_t> &plainData, CryptFileType cfType);
	bool decryptFileData(const std::vector<uint8_t> &cryptedData, std::vector<uint8_t> &plainData, CryptFileType cfType);

	static bool cryptFilesExist();
	bool cryptFilesCheckMasterPass(const wxString &masterPass);
	void cryptFilesCreate(const wxString &masterPass);
	void cryptFilesChangeMasterPass(const wxString &oldMasterPass, const wxString &newMasterPass);

	static void deleteCryptedFile(CryptFileType cfType);
	
	static bool getFilePathByType(CryptFileType cfType, wxString &filePath);

protected:
	static std::string getFileNameByType(CryptFileType cfType);
	AesFile::KeyFile& getKeyFile(CryptFileType cfType);
	void generateKeyFile(ShaMasterPass &mpass, CryptFileType cfType);
	void generateNewKeys(const wxString & pass);

	void setNewPassword(const wxString &pass);
	
	bool checkCryptedFile(CryptFileType cfType);
	
	static bool isFileExists(const wxString &filePath);
	static void deleteAllCryptedFiles();

	void clearCryptedFile(CryptFileType cfType);
	void changePassOnFile(CryptFileType cfType, const wxString &oldMasterPass, const wxString &newMasterPass);

	static wxString getRedirectedName(const wxString &filePath);
	

private:
	
	// AES-256 Keys based on Master Password Hash
	AesFile::KeyFile keyFileNull;
	AesFile::KeyFile keyFileZillaXml;      // "filezilla.xml"
	AesFile::KeyFile keySiteManagerXml;    // "sitemanager.xml"
	AesFile::KeyFile keyRecentServersXml;  // "recentservers.xml"	
	AesFile::KeyFile keyQueueSqlite3;      // "queue.sqlite3"
	AesFile::KeyFile keyQueueXml;          // "queue.xml"
	AesFile::KeyFile keyBookmarksXml;	   // "bookmarks.xml"
	AesFile::KeyFile keyTrustedcertsXml;   // "trustedcerts.xml"

	//bool testCryptFileData();
};

#endif
