#include "CryptedDataBaseFile.h"
#include "CryptFiles.h"
#include <filezilla.h>
#include "queue.h"
#include "Mainfrm.h"
#include "Options.h"


CryptedDataBaseFile::CryptedDataBaseFile(CryptFileType cfType)
	: mCfType(cfType)
{
}

CryptedDataBaseFile::~CryptedDataBaseFile()
{
}

bool CryptedDataBaseFile::saveRawDataToFile(const std::vector<uint8_t>& data)
{
	return saveFile(data);
}

bool CryptedDataBaseFile::loadRawDataAndClearFile(std::vector<uint8_t>& data)
{
	data.clear();

	if (!loadFile(data))
		data.clear();

	return clearFile();
}

bool CryptedDataBaseFile::saveFile(const std::vector<uint8_t>& data)
{
	// Saving is not allowed at OPTION_DEFAULT_KIOSKMODE = 2
	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
		return true;

	std::vector<uint8_t> crypted_data = CryptFiles::instance().encryptFileData(data, mCfType);
	return CryptFiles::saveAllDataToFile(mCfType, crypted_data);
}

bool CryptedDataBaseFile::clearFile()
{
	std::vector<uint8_t> empty_data;
	return saveFile(empty_data);
}

bool CryptedDataBaseFile::loadFile(std::vector<uint8_t>& data)
{
	data.clear();

	std::vector<uint8_t> crypted_data;
	if (CryptFiles::loadAllDataFromFile(mCfType, crypted_data))
	{
		if (CryptFiles::instance().decryptFileData(crypted_data, data, mCfType))
		{
			return true;
		}
		else
			data.clear();
	}

	return false;
}
