#include "MasterPassword.h"
#include "Common.h"
#include <wx/textdlg.h>
#include "CryptFiles.h"
#include "CMasterPassCreateDialog.h"
#include "CMasterPassEnterDialog.h"
#include "CMasterPassChangeDialog.h"
#include <filezilla.h>
#include "queue.h"
#include "Mainfrm.h"
#include "Options.h"
#include "filezillaapp.h"
#include "ipcmutex.h"
#include <wx/log.h>
#include <wx/file.h>
#include <wx/filename.h>



bool IsAppUniqueTryLock()
{
	static CInterProcessMutex GlobalMutex(MUTEX_APP_GLOBAL, false);
	return GlobalMutex.TryLock() == 1;
}

bool waitAppUniqueTryLock()
{
	bool res = false;

	while (true)
	{
		if (IsAppUniqueTryLock())
		{
			res = true;
			break;
		}
		else
		{
			const int msg_ret = wxMessageBoxEx(	_T("The another instance of FileZillaSecure is open.\nPlease close it and press OK to continue."), 
												_T("Close the other instances of FileZillaSecure"), 
												wxOK | wxCANCEL | wxCENTRE | wxICON_WARNING);

			if (msg_ret != wxOK)
			{
				res = false;
				break;
			}
		}
	}

	return res;
}

bool NewMasterPassword(bool isAppUniq)
{
	bool res = false;
	
	wxString pass;
	if (showDlgNewMasterPassword(pass))
	{
		bool isLock = false;

		if (isAppUniq)
			isLock = true;
		else
		{
			if (waitAppUniqueTryLock())
				isLock = true;
		}

		if (isLock)
		{
			CryptFiles::instance().cryptFilesCreate(pass);
			res = true;
			wxMessageBoxEx(_T("The New Password has set."), _T("New Master Password"), wxOK | wxCENTRE | wxICON_INFORMATION);
		}
	}

	return res;
}

bool ChangeMasterPassword(bool isAppUniq)
{
	bool res = false;

	while (true)
	{
		wxString oldPass;
		wxString newPass;
		if (showDlgChangeMasterPassword(oldPass, newPass))
		{
			if (CryptFiles::instance().cryptFilesCheckMasterPass(oldPass))
			{
				bool isLock = false;

				if (isAppUniq)
					isLock = true;
				else
				{
					if (waitAppUniqueTryLock())
						isLock = true;
				}

				if (isLock)
				{
					CryptFiles::instance().cryptFilesChangeMasterPass(oldPass, newPass);
					res = true;
					wxMessageBoxEx(_T("The Master Password has changed."), _T("Change Master Password"), wxOK | wxCENTRE | wxICON_INFORMATION);
				}

				break;
			}
			else
				wxMessageBoxEx(_T("The Old Password is invalid.\nTry again."), _T("Change Master Password"), wxOK | wxCENTRE | wxICON_WARNING);
		}
		else
			break;
	}

	return res;
}

bool EnterOrChangeMasterPassword(bool isAppUniq)
{
	bool res = false;

	while (true)
	{
		wxString enterPass;
		bool isChangePass = false;
		if (showDlgEnterMasterPassword(enterPass, isChangePass))
		{
			if (isChangePass)
			{
				if (ChangeMasterPassword(isAppUniq))
					res = true;

				break;
			}
			else  // Enter Pass
			{
				if (CryptFiles::instance().cryptFilesCheckMasterPass(enterPass))
				{
					res = true;
					break;
				}
				else
					wxMessageBoxEx(_T("The Master Password is invalid.\nTry again."), _T("Enter Master Password"), wxOK | wxCENTRE | wxICON_WARNING);
			}
		}
		else
			break;
	}

	return res;
}


void CreateSettingsDir()
{
	wxLogNull logNo;
	
	CLocalPath p = COptions::GetUnadjustedSettingsDir();
	if (!p.empty() && !p.Exists())
		wxFileName::Mkdir( p.GetPath(), 0700, wxPATH_MKDIR_FULL );
}


// That function calls first when the app runs.
// return: true -> Continue App Loading
// return: false -> Exit App
bool MasterPasswordLoop()
{
	CreateSettingsDir();			

	const bool isAppUniq = IsAppUniqueTryLock();

	bool res = false;

	if (!CryptFiles::cryptFilesExist())  // Crypto Container Not Found
	{
		if (NewMasterPassword(isAppUniq))
			res = true;
	}
	else  // Ctypto Container Found
	{
		if (EnterOrChangeMasterPassword(isAppUniq))
			res = true;
	}

	return res;
}
