#include "ShaMasterPass.h"
#include "msgbox.h"
#include <wx/buffer.h>
#include <stdio.h>
#include "mbedtls/sha512.h"
#include "Common.h"


void ShaMasterPass::calcSHA512(const uint8_t *data, size_t data_size, uint8_t out_hash[64])
{
	/* NETTLE SOLUTION
	sha512_ctx ctx = {};
	sha512_init(&ctx);
	sha512_update(&ctx, data_size, data);
	sha512_digest(&ctx,	hash_size, hash);
	*/

	// MBEDTLS solution
	/**
	* \brief          Output = SHA-512( input buffer )
	*
	* \param input    buffer holding the  data
	* \param ilen     length of the input data
	* \param output   SHA-384/512 checksum result
	* \param is384    0 = use SHA512, 1 = use SHA384
	*/
	mbedtls_sha512(data, data_size, out_hash, 0);
}

ShaMasterPass::ShaMasterPass(const wxString &pass)
{
	generateMasterPassHash(pass);
//	showDebugMsg(wxString(_T("MasterHashSHA512=")) + arrayToHex(sha512MasterPass, sizeof(sha512MasterPass)) + wxString(_T("\n")));
}

ShaMasterPass::~ShaMasterPass()
{
}

AesFile::KeyFile ShaMasterPass::generateKeyForFile(const std::string & fileName)
{
	AesFile::KeyFile keyFile = {};

	const char *str_ptr = fileName.c_str();
	const size_t str_size = fileName.size();

	const size_t size_buf = sizeof(sha512MasterPass) + str_size;
	uint8_t *buf = new uint8_t[size_buf];
	{
		memcpy(buf, sha512MasterPass, sizeof(sha512MasterPass));
		memcpy(buf + sizeof(sha512MasterPass), str_ptr, str_size);

		uint8_t hash[64] = {};
		calcSHA512(buf, size_buf, hash);

		uint8_t out_key[32] = {};
		uint8_t out_iv[16] = {};
		memcpy(out_key, hash, sizeof(out_key));
		memcpy(out_iv, hash + sizeof(out_key), sizeof(out_iv));

		uint8_t hash_fname[64] = {};
		uint8_t magic[16] = {};
		calcSHA512((const uint8_t*)str_ptr, str_size, hash_fname);
		memcpy(magic, hash_fname, sizeof(magic));

		keyFile.fname = fileName;
		memcpy(keyFile.magic, magic, sizeof(keyFile.magic));
		memcpy(keyFile.key, out_key, sizeof(keyFile.key));
		memcpy(keyFile.iv, out_iv, sizeof(keyFile.iv));
	}
	memset(buf, 0, size_buf);
	delete[] buf;

	return keyFile;
}

void ShaMasterPass::generateMasterPassHash(const wxString &pass)
{
	memset(sha512MasterPass, 0, sizeof(sha512MasterPass));

	wxCharBuffer chbuf = pass.utf8_str();
	const uint8_t *buf = (uint8_t *)chbuf.data();
	const size_t len = (size_t)chbuf.length();

	calcSHA512(buf, len, sha512MasterPass);
}
