#ifndef __CRYQUEUE_STORAGE_IMPLEMENTATION_H__
#define __CRYQUEUE_STORAGE_IMPLEMENTATION_H__


#include <vector>
#include <cstdint>
#include <wx/string.h>
#include <wx/strconv.h>
#include <wx/hashmap.h>
#include <unordered_map>
#include <map>
#include "CryptedQueueDataBase.h"


class CServer;
class CServerItem;
class ServerElement;
class CFileItem;
class CFolderItem;
class CLocalPath;
class CServerPath;


class CCryptedQueueStorageImpl
{
public:
	explicit CCryptedQueueStorageImpl();
	virtual ~CCryptedQueueStorageImpl();

	bool saveQueue(std::vector<CServerItem*> const& queue);
	bool loadQueue(std::vector<ServerElement> &queue);

protected:
	bool saveQueueToDB(std::vector<CServerItem*> const& queue, CryptedQueueDataBase& cDB);
	bool loadQueueFromDB(std::vector<ServerElement> &queue, CryptedQueueDataBase& cDB);
	
	void ReadLocalPaths(CryptedQueueDataBase & cDB);
	void ReadRemotePaths(CryptedQueueDataBase & cDB);

	const CLocalPath& GetLocalPath(int64_t id) const;
	const CServerPath& GetRemotePath(int64_t id) const;
	
	// Caches to speed up saving and loading
	void ClearCaches();

	int64_t SaveLocalPath(const CLocalPath& path, CryptedQueueDataBase & cDB);
	int64_t SaveRemotePath(const CServerPath& path, CryptedQueueDataBase & cDB);

	bool SaveFile(const CFileItem& item, int64_t server_id, CryptedQueueDataBase & cDB);
	bool SaveDirectory(const CFolderItem& item, int64_t server_id, CryptedQueueDataBase & cDB);
	bool SaveServer(const CServerItem& item, CryptedQueueDataBase& cDB);
	bool serverTableAddServer(const CServer& server, int64_t &server_id, CryptedQueueDataBase & cDB);

	int64_t ParseServerFromRow(CServer& server, const CDBTableServerLine &serverLine);
	int64_t ParseFileFromRow(CFileItem** pItem, const CDBTableFileLine &fileLine);

private:

	struct fast_equal
	{
		inline bool operator()(wxString const& lhs, wxString const& rhs) const
		{
			return lhs == rhs;
		}
	};

	std::unordered_map<wxString, int64_t, wxStringHash, fast_equal> localPaths_;
	std::unordered_map<wxString, int64_t, wxStringHash> remotePaths_; // No need for fast_equal as GetSafePath returns unshared string anyhow

	std::map<int64_t, CLocalPath> reverseLocalPaths_;
	std::map<int64_t, CServerPath> reverseRemotePaths_;
};


#endif //__CRYQUEUE_STORAGE_IMPLEMENTATION_H__
