#ifndef __CRYXMLFUNCTIONS_H__
#define __CRYXMLFUNCTIONS_H__

#ifdef HAVE_LIBPUGIXML
#include <pugixml.hpp>
#else
#include "../pugixml/pugixml.hpp"
#endif

#include "CryptFiles.h"


class CCryptedXmlFile final
{
public:
	explicit CCryptedXmlFile(CryptFileType cfType, wxString const& root = wxString());

	~CCryptedXmlFile();

	CCryptedXmlFile(CCryptedXmlFile const&) = delete;
	CCryptedXmlFile& operator=(CCryptedXmlFile const&) = delete;

	pugi::xml_node CreateEmpty();

	wxString GetFileName() const { return m_fileName; }

	// Sets error description on failure
	pugi::xml_node Load();

	wxString GetError() const { return m_error; }
	size_t GetRawDataLength();
	void GetRawDataHere(char* p, size_t size); // p has to big enough to hold at least GetRawDataLength() bytes

	bool ParseData(char* data); // data has to be 0-terminated

	void Close();

	pugi::xml_node GetElement() { return m_element; }
	pugi::xml_node const GetElement() const { return m_element; }

	bool Modified();

	bool Save(bool printError);

	bool IsFromFutureVersion() const;

protected:

	void SetFileName(const wxString& name);
	bool HasFileName() const { return !m_fileName.empty(); }



	// Opens the specified XML file if it exists or creates a new one otherwise.
	// Returns 0 on error.
	bool GetXmlFile(wxString const& file);

	// Sets version and platform in root element
	void UpdateMetadata();

	// Save the XML document to the given file
	bool SaveXmlFile();

	fz::datetime m_modificationTime;
	wxString m_fileName;
	pugi::xml_document m_document;
	pugi::xml_node m_element;

	wxString m_error;

	wxString m_rootName{_T("FileZilla3")};

	CryptFileType mCryptedFileType;
};


#endif //__CRYXMLFUNCTIONS_H__
